package junit.extensions.eclipse.diff;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.eclipse.jdt.internal.junit.ui.TestRunInfo;

public class JUnitDiff {
    private final static String REGEXP_EQUALS_ERROR 
        = ".+: (?:(.*) )?expected:<(.*)> but was:<(.*)>\\s+at.*";
    private final static String REGEXP_SAME_ERROR 
        = ".+: (?:(.*) )?expected same:<(.*)> was not:<(.*)>\\s+at.*";
    private final static Pattern[] ERROR_PATTERNS = {
        Pattern.compile(REGEXP_EQUALS_ERROR, Pattern.MULTILINE | Pattern.DOTALL),
        Pattern.compile(REGEXP_SAME_ERROR, Pattern.MULTILINE | Pattern.DOTALL),
    };

    private String testName;
    private String expected;
    private String actual;
    private String message;

    public JUnitDiff(TestRunInfo testRunInfo) {
        if (testRunInfo == null)
            return;
        testName = testRunInfo.getTestName();
        scanTrace(testRunInfo.getTrace());
    }

    public String getTestName() {
        return testName;
    }

    public String getExpected() {
        return expected;
    }

    public String getActual() {
        return actual;
    }

    public String getMessage() {
        return message;
    }

    private void scanTrace(String trace) {
        if (trace == null)
            return;
        for (int i = 0; i < ERROR_PATTERNS.length; ++i) {
            Pattern p = ERROR_PATTERNS[i];
            Matcher m = p.matcher(trace);
            if (m.matches()) {
                message = m.group(1);
                expected = m.group(2);
                actual = m.group(3);
                break;
            }
        }
    }

    private boolean equalsSafe(Object o1, Object o2) {
        if (o1 == null && o2 != null)
            return false;
        if (o1 != null && o2 == null)
            return false;
        if (o1 == null && o2 == null)
            return true;
        return o1.equals(o2);
    }

    public boolean equals(Object o) {
        if (o == this)
            return true;
        if (!(o instanceof JUnitDiff))
            return false;
        JUnitDiff diff = (JUnitDiff) o;
        return equalsSafe(testName, diff.testName)
            && equalsSafe(expected, diff.expected)
            && equalsSafe(actual, diff.actual)
            && equalsSafe(message, diff.message);
    }
    
    private int getHashCode(Object o) {
        return o == null ? 0 : o.hashCode();
    }

    public int hashCode() {
        int result = 17;
        result = 37*result + getHashCode(testName);
        result = 37*result + getHashCode(expected);
        result = 37*result + getHashCode(actual);
        result = 37*result + getHashCode(message);
        return result;
    }

}

