package org.eclipse.jdt.internal.junit.ui;

import java.lang.reflect.Field;

import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.events.SelectionListener;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Table;
import org.eclipse.swt.widgets.Tree;

abstract class TestIdSelectionBroadcaster {
    protected Object view;
    private Control control;
    
    private ITestIdSelectionListener testIdSelectionListener;
    private ControlListener listener;

    public TestIdSelectionBroadcaster(Object view, Class klass, String controlName) {
        this.view = view;
        control = getControl(klass, controlName);
        listener = new ControlListener();
        beginListening();
    }

    public void dispose() {
        endListening();
    }

    public void setTestIdSelectionListener(ITestIdSelectionListener listener) {
        this.testIdSelectionListener = listener;
    }

    private void beginListening() {
        if (control == null)
            return;
        control.addMouseListener(listener);
        if (control instanceof Tree)
             ((Tree) control).addSelectionListener(listener);
        else if (control instanceof Table)
             ((Table) control).addSelectionListener(listener);
    }

    private void endListening() {
        if (control == null || control.isDisposed())
            return;
        control.removeMouseListener(listener);
        if (control instanceof Tree)
            ((Tree) control).removeSelectionListener(listener);
        else if (control instanceof Table)
            ((Table) control).removeSelectionListener(listener);
    }

    protected Control getControl(Class klass, String controlName) {
        if (view == null)
            return null;
        try {
            Field f = klass.getDeclaredField(controlName);
            f.setAccessible(true);
            return (Control) f.get(view);
        } catch (Throwable e) {
            return null;
        }
    }

    abstract protected String getSelectedTestId();

    private void testIdChanged() {
        if (testIdSelectionListener != null) {
            testIdSelectionListener.testIdChanged(getSelectedTestId());
        }
    }

    private class ControlListener extends MouseAdapter implements SelectionListener {
        public void widgetSelected(SelectionEvent e) {
            testIdChanged();
        }
        public void widgetDefaultSelected(SelectionEvent e) {
            testIdChanged();
        }
        public void mouseDown(MouseEvent e) {
            testIdChanged();
        }
        public void mouseUp(MouseEvent e) {
            testIdChanged();
        }
    }
}
